
import React from 'react';
import ReactDOM from 'react-dom/client';
import { BrowserRouter, Route, Routes } from 'react-router-dom';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import App from './App.tsx';
import './index.css';
import { initPerformanceMonitoring } from './lib/performance-monitor';
import { addSecurityHeadersToDocument } from './lib/security-headers';
import { hasAnalyticsConsent, initializeAnalytics } from './lib/analytics';
import { PageErrorBoundary } from './components/PageErrorBoundary';
import { initializeStorage } from './lib/storage-helpers';

// Initialize performance monitoring in development
initPerformanceMonitoring();

// Add security headers to the document
addSecurityHeadersToDocument();

// Always initialize Google Analytics and let the consent system handle permission
// This ensures the GA scripts are loaded even if consent hasn't been granted yet
initializeAnalytics('G-7QM6MVB1E3');

// Initialize storage early
initializeStorage().catch(err => {
  console.error('Failed to initialize storage:', err);
});

// Create a client with optimized settings for performance
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      retry: 1, // Reduce retry count to improve perceived performance
      refetchOnWindowFocus: false,
      staleTime: 5 * 60 * 1000, // 5 minutes
      gcTime: 10 * 60 * 1000, // 10 minutes (formerly cacheTime in v4)
      meta: {
        // Use meta to configure error handling
        onError: (error: Error) => {
          console.error('Query error:', error);
        }
      }
    },
  },
});

// This helps with handling direct URL requests with refresh
const handleDirectAccess = () => {
  // If we get a direct non-root URL request, make sure our router handles it
  const path = window.location.pathname;
  if (path !== '/' && !path.includes('.')) {
    // This is a route, not an asset
    console.log("Direct route access detected:", path);
    
    // For profile routes, ensure they're handled correctly
    if (path.startsWith('/profile') || path.startsWith('/profiles')) {
      console.log("Profile route detected:", path);
      
      // Add a class to help with special CSS handling if needed
      document.body.classList.add('profile-route');
    }
  }
};

// Enhanced MIME type and module loading error handling
const handleMimeTypeErrors = () => {
  window.addEventListener('error', function(e) {
    const errorMessage = e.message || '';
    const errorFilename = e.filename || '';
    
    if (errorMessage.includes('MIME type') || 
        errorMessage.includes('loading dynamically imported module')) {
      
      console.error('Module loading error detected:', {
        message: errorMessage,
        filename: errorFilename,
        lineno: e.lineno,
        colno: e.colno,
        timestamp: new Date().toISOString()
      });
      
      // Advanced fallback mechanism
      if (errorFilename.includes('.tsx') || errorFilename.includes('.jsx')) {
        console.log('Attempting advanced module loading fallback:', errorFilename);
        
        // Attempt multiple fallback strategies
        const fallbackStrategies = [
          () => window.moduleLoadFallback?.(errorFilename),
          () => import(/* @vite-ignore */ errorFilename),
          () => fetch(errorFilename).then(response => response.text())
        ];
        
        const attemptFallbacks = async () => {
          for (const strategy of fallbackStrategies) {
            try {
              await strategy();
              console.log('Module loaded successfully via strategy');
              return;
            } catch (err) {
              console.warn('Fallback strategy failed:', err);
            }
          }
          
          // Final error reporting
          window.moduleLoadErrorHandler?.(
            new Error('All module loading strategies failed'), 
            errorFilename
          );
        };
        
        attemptFallbacks();
      }
    }
  }, true);
};

// Call our enhanced error handling setup
handleMimeTypeErrors();

// Call this to handle direct access
handleDirectAccess();

// Optimize initial rendering
const preconnectToOrigins = () => {
  const origins = [
    'https://travilis.com',
    'https://eyedvzmodfhyrxnhhrvf.supabase.co',
    'https://images.unsplash.com',
    'https://www.googletagmanager.com'
  ];
  
  origins.forEach(origin => {
    const link = document.createElement('link');
    link.rel = 'preconnect';
    link.href = origin;
    link.crossOrigin = 'anonymous';
    document.head.appendChild(link);
  });
};

// Set up preconnect links
preconnectToOrigins();

// Make sure the root element exists
const rootElement = document.getElementById('root');
if (!rootElement) {
  console.error('Root element not found');
} else {
  ReactDOM.createRoot(rootElement).render(
    <React.StrictMode>
      <PageErrorBoundary>
        <QueryClientProvider client={queryClient}>
          <BrowserRouter basename="/">
            <App />
          </BrowserRouter>
        </QueryClientProvider>
      </PageErrorBoundary>
    </React.StrictMode>,
  );
}
